#!/usr/bin/env python
from plumbum import FG
from plumbum.machines import SshMachine

def patch_plumbum():
    import os.path
    from plumbum.commands.base import BaseCommand, shquote
    from plumbum.machines import SshMachine
    from plumbum.machines.session import MarkedPipe, ShellSession
    from plumbum.path import RemotePath

    shell_cmd = ["sudo", "su", "root", "-c"]

    def patch_ssh_machine_popen(self, args, ssh_opts = (), **kwargs):
        cmdline = []
        cmdline.extend(ssh_opts)
        cmdline.append(self._fqhost)
        sh_cmdline = []
        if shell_cmd:
            def formulate(arg):
                if isinstance(arg, BaseCommand):
                    return arg.formulate()
                return (shquote(arg), )
            quote = shquote
        else:
            formulate = quote = lambda x: x
        if args:
            if hasattr(self, 'env'):
                envdelta = self.env.getdelta()
                sh_cmdline.extend(['cd', quote(str(self.cwd)), '&&'])
                if envdelta:
                    sh_cmdline.append('env')
                    sh_cmdline.extend('%s=%s' % (quote(k), quote(v)) for k, v in envdelta.items())
            if isinstance(args, (tuple, list)):
                for arg in args:
                    sh_cmdline.extend(formulate(arg))
            else:
                sh_cmdline.extend(formulate(args))
        if shell_cmd:
            cmdline.extend(shell_cmd)
            if sh_cmdline:
                cmdline.append(quote(' '.join(sh_cmdline)))
        else:
            cmdline.extend(sh_cmdline)
        return self._ssh_command[tuple(cmdline)].popen(**kwargs)

    @property
    def patch_remote_path_suffix(self):
        return os.path.splitext(str(self))[1]

    SshMachine.popen = patch_ssh_machine_popen
    RemotePath.suffix = patch_remote_path_suffix

def main():
    with SshMachine("azure") as azure:
        pkg = azure["pkg"]
        freebsd_update = azure["freebsd-update"]["--not-running-from-cron"]

        pkg["upgrade", "-y"] & FG
        pkg["install", "-yU", "lighttpd", "poudriere", "vim-console"] & FG
        freebsd_update["fetch"] & FG
        with azure.env(PAGER="cat"):
            freebsd_update["install"] & FG

if __name__ == "__main__":
    patch_plumbum()
    main()
